/*
 * An opener for a file provided by an Ext2-compatible filesystem.
 *
 * Copyright (C) 2021 Paul Boddie <paul@boddie.org.uk>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA  02110-1301, USA
 */

#pragma once

#include <ext2fs/ext2fs.h>

#include <fsclient/file.h>
#include <fsserver/ext2_file_operations.h>
#include <fsserver/opener_resource.h>
#include <systypes/user.h>



/* Forward declarations. */

struct Ext2FileOpenerDir;



/* Support for providing access to files. */

class Ext2FileOpener : public OpenerResource
{
protected:
    Ext2FileOperations *_ops;
    user_t _user;

    /* Convenience methods determining different object types. */

    virtual bool accessing_directory(const char *path, flags_t flags, fileid_t fileid);

    virtual bool accessing_file(const char *path, flags_t flags, fileid_t fileid);

    /* Convenience methods obtaining different pager types. */

    virtual long get_directory(const char *path, flags_t flags, fileid_t fileid, offset_t *size, l4_cap_idx_t *cap);

public:
    explicit Ext2FileOpener(FilePaging *paging, Ext2FileOperations *ops, user_t user)
    : OpenerResource(paging), _ops(ops), _user(user)
    {
    }

    virtual ~Ext2FileOpener();

    /* Configurable methods. */

    virtual long get_fileid(const char *path, flags_t flags, fileid_t *fileid);

    virtual long make_accessor(const char *path, flags_t flags, fileid_t fileid, Accessor **accessor);

    /* Helper methods. */

    void read_directory(fileid_t fileid, file_t *writer);

    int write_directory_entry(struct Ext2FileOpenerDir *dir);
};



/* Helper structures. */

struct Ext2FileOpenerDir
{
    Ext2FileOpener *opener;
    file_t *writer;
    struct ext2_dir_entry *entry;
    long offset;
};

// vim: tabstop=4 expandtab shiftwidth=4
